local addonName, addon = ...

-- functions
local format = string.format
local unpack, wipe = unpack, wipe
-- constants
-- frames
local GameTooltip = GameTooltip
-- libs
local LibStub = LibStub

addon.broker = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(addonName, {
    type = "data source",
    label = "Keystone",
    text = "No Keystone",
    icon = [[Interface\Icons\inv_relics_hourglass]],
})

local function GetTooltipLinesForPool(self, pool)
    if not pool then return {} end

    local lines = {}

    -- local character, keystone
    -- local instanceName, level, isDepleted
    for _, character in pairs(pool) do
        local realm, faction, name, class, guild, keystones, records = unpack(character)
        if keystones and #keystones > 0 then
            for i = 1, #keystones do
                local _, _, instanceName, level, isDepleted = unpack(keystones[i])

                if isDepleted then
                    table.insert(lines, {
                        format("%s", self:GetCharacterDisplayName(false, unpack(character))),
                        format("%s %s", instanceName, level),
                        0.4, 0.4, 0.4,
                        0.4, 0.4, 0.4,
                    })
                else
                    table.insert(lines, {
                        format("%s", self:GetCharacterDisplayName(true, unpack(character))),
                        format("%s |cff00FF00%s|r", instanceName, level),
                        1.0, 1.0, 1.0,
                        1.0, 1.0, 1.0,
                    })
                end
            end
        end
    end

    return lines
end

local function GetTooltipLines(self)
    -- todo: cleanup

    local lines = {}
    if self.debug then
        table.insert(lines, {"Debug mode enabled", nil, 1, 0, 0})
    end

    local pool, poolLines = {}, {}

    -- our own keystones
    pool = self.db.pools["player"]
    if pool then
        wipe(poolLines)
        poolLines = GetTooltipLinesForPool(self, pool)
        if #poolLines > 0 then
            if #lines > 0 then table.insert(lines, " ") end
            table.insert(lines, {
                "Character",
                format("%d |cffFFFFFF%s|r", #poolLines, (#poolLines == 1 and "Keystone" or "Keystones")),
                1.0, 1.0, 1.0,
            })
            table.insert(lines, " ")
            for i = 0, #poolLines do
                table.insert(lines, poolLines[i])
            end
        end
    end

    -- party members' keystones
    pool = self.db.pools["Party"]
    if pool then
        if #lines > 0 then table.insert(lines, " ") end

        if not IsControlKeyDown() then
            table.insert(lines, {
                "Party",
                "[Control] to view",
                0.66, 0.66, 1.0,
                0.6, 0.6, 0.6,
            })
        else
            wipe(poolLines)
            poolLines = GetTooltipLinesForPool(self, pool)
            if #poolLines > 0 then
                table.insert(lines, {
                    "Party",
                    format("%d |cffFFFFFF%s|r", #poolLines, (#poolLines == 1 and "Keystone" or "Keystones")),
                    0.66, 0.66, 1.0,
                })
                table.insert(lines, " ")
                for i = 0, #poolLines do
                    table.insert(lines, poolLines[i])
                end
            end
        end
    end

    -- guild members' keystones
    pool = self.db.pools[format(
        "Guild.%s.%s.%s", self.player.guild.realm or "", self.player.faction or "", self.player.guild.name or ""
    )]
    if pool then
        if #lines > 0 then table.insert(lines, " ") end

        if not IsShiftKeyDown() then
            table.insert(lines, {
                self.player.guild.name,
                "[Shift] to view",
                0.2, 1.0, 0.0,
                0.6, 0.6, 0.6,
            })
        else
            wipe(poolLines)
            poolLines = GetTooltipLinesForPool(self, pool)
            if #poolLines > 0 then
                table.insert(lines, {
                    self.player.guild.name,
                    format("%d |cffFFFFFF%s|r", #poolLines, (#poolLines == 1 and "Keystone" or "Keystones")),
                    0.25, 1.0, 0.0,
                })
                table.insert(lines, " ")
                for i = 0, #poolLines do
                    table.insert(lines, poolLines[i])
                end
            end
        end
    end

    if #lines < 1 then
        table.insert(lines, {"No Keystones", nil, 1.0, 1.0, 1.0})
        return lines
    end

    return lines
end

function addon.broker:OnTooltipShow()
    local lines = GetTooltipLines(addon)

    for _, line in pairs(lines) do
        if type(line) == "string" then
            line = {line}
        end

        self:AddDoubleLine(unpack(line))
    end
end

function addon.broker:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("BOTTOM", self, "BOTTOM")
    GameTooltip:ClearLines()
    addon.broker.OnTooltipShow(GameTooltip)
    GameTooltip:Show()
end

function addon.broker:OnLeave()
    GameTooltip:Hide()
end

function addon.broker:NotifyChange(keystone)
    if not keystone then return end

    local _, _, instanceName, level, isDepleted = unpack(keystone)

    if isDepleted then
        self.text = format("|cff666666%s %s|r", instanceName, level)
    else
        self.text = format("|cffFFFFFF%s|r |cff00FF00%s|r", instanceName, level)
    end
end
