local _, addon = ...

-- functions
local format = string.format
local CreateFrame = CreateFrame
local GetGuildInfo = GetGuildInfo
local GetRealmName = GetRealmName
local GuildRoster = GuildRoster
local UnitClass = UnitClass
local UnitFactionGroup = UnitFactionGroup
local UnitName = UnitName
-- constants
-- frames
-- libs

function addon:Initialise(db)
    self:Print("Initialising.")

    local unit = "player"

    self.db = db
    self.debug = self.db.debug and self.db.debug == true
    if not self.db.pools then self.db.pools = {} end
    if not self.db.pools[unit] then self.db.pools[unit] = {} end

    local name = select(1, UnitName(unit))
    self.player = {
        name = name or "",
        realm = GetRealmName() or "",
        class = select(2, UnitClass(unit)) or "",
        faction = select(2, UnitFactionGroup(unit)) or "",
        guild = {
            name = nil,
            realm = nil,
        },
    }
    self:Print("New player info: `%s.%s.%s`", self.player.realm, self.player.faction, self.player.name)

    -- listen for guild changes
    do
        local f = CreateFrame("frame")
        f:SetScript("OnEvent", function(_, event)
            self:Print("Event: `%s`", event)

            local guildName, guildRank, _, guildRealm = GetGuildInfo(unit)
            if not guildName then return end

            self.player.guild.name = guildName
            self.player.guild.realm = guildRealm or self.player.realm

            self:Print("New guild info: `%s.%s.%s`", self.player.guild.realm, self.player.faction, self.player.guild.name)
        end)
        f:RegisterEvent("GUILD_MOTD")
        --f:RegisterEvent("GUILD_ROSTER_UPDATE")
        f:RegisterEvent("PLAYER_GUILD_UPDATE")
        GuildRoster()
    end

    if self.InitialiseComms then
        self:InitialiseComms()
    end
end

function addon:Print(obj, ...)
    if not self.debug then return end

    if obj and type(obj) == "table" then
        print(format("|cff%sBK:|r", "6699CC"))
        DevTools_Dump(obj)
        return
    end

    print(format("|cff%sBK:|r %s", "6699CC", type(obj) == "string" and format(obj, ...) or tostring(obj)))
end

function addon:NotifyChange(pool, character)
    local realm, faction, name, class, guild, keystones, records = unpack(character)

    if self:CharacterIsMe(character) then
        local keystone = keystones and #keystones > 0 and keystones[1] or nil
        self.broker:NotifyChange(keystone)
    end
end
